//*********************************************************************************
//*                                                                               *
//*  Unit Description : Hardware Depandant Shareware Lock                         *
//*                                                                               *
//*  Author/Developer : Richard de Jong                                           *
//*                                                                               *
//*  Copyright (c) Richard de Jong                                                *
//*  All Rights reserved.                                                         *
//*                                                                               *
//*  19-10-99 11:43:30                                                            *
//*                                                                               *
//*  This unit is a wrapper for HSLOCK.DLL                                        *
//*                                                                               *
//*  This unit provides also a component                                          *
//*  editor.                                                                      *
//*                                                                               *
//*                                                                               *
//*                                                                               *
//*********************************************************************************
unit HSLOCK;

interface

uses
  Classes,
  Forms,
  IniFiles,
  Windows,
  SysUtils,
  DsgnIntf;

{ Unit-wide declarations }

var
amf:String;
InitOk:Boolean;

type
  TShareWare = class(TComponent)
    private
      { Private fields of TShareWare }
        FOrderFile:String;
        { Storage for property ComputerID }
        FComputerID : String;
        { Storage for property NrOfDays }
        FNrOfDays : Integer;
        { Storage for property ProgramName }
        FProgramName : String;
        { Storage for property ProgramUniqueKey }
        FProgramUniqueKey : String;
        { Storage for property ProgramVersion }
        FProgramVersion : String;

        FRName,FRID,FRCODE:String;
      { Private methods of TShareWare }
        { Method to set variable and property values and create objects }
        procedure AutoInitialize;
        { Method to free any objects created by AutoInitialize }
        procedure AutoDestroy;
        { Read method for property ComputerID }
        function GetComputerID : String;
        { Read method for property ProgramName }
        function GetProgramName : String;
        { Write method for property ProgramName }
        procedure SetProgramName(Value : String);
        { Read method for property ProgramUniqueKey }
        function GetProgramUniqueKey : String;
        { Write method for property ProgramUniqueKey }
        procedure SetProgramUniqueKey(Value : String);
        { Read method for property ProgramVersion }
        function GetProgramVersion : String;
        { Write method for property ProgramVersion }
        procedure SetProgramVersion(Value : String);

    protected
      { Protected fields of TShareWare }

      { Protected methods of TShareWare }
        procedure Loaded; override;

    public
      { Public fields and properties of TShareWare }
        { The Id of the current computer. The user must report this key. }
        property ComputerID : String read GetComputerID;

      { Public methods of TShareWare }
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
        function Execute : Boolean;

    published
      { Published properties of TShareWare }
        { Number of days until expiration }
        property RegName:string read FRName write FRName;
        property RegID:string read FRID write FRID;
        property RegCODE:string read FRCode write FRCode;
        property NrOfDays : Integer read FNrOfDays write FNrOfDays default 30;
        property OrderFileLocation: String read FOrderFile write FOrderFile;
        { The name of the program without the version number }
        property ProgramName : String read GetProgramName write SetProgramName;
        { A unique key, Example: A4495B. Use a different key in other versi }
        property ProgramUniqueKey : String
             read GetProgramUniqueKey write SetProgramUniqueKey;
        { The Version of the program. }
        property ProgramVersion : String
             read GetProgramVersion write SetProgramVersion;

  end;

type TShareEditor = Class(TComponentEditor)
public
function GetVerbCount:Integer;override;
function GetVerb(index:integer):String;override;
procedure ExecuteVerb(Index:Integer);override;
end;

procedure Register;

implementation

procedure Register;
begin
     { Register TShareWare with System as its
       default page on the Delphi component palette }
     RegisterComponents('System', [TShareWare]);
     RegisterComponentEditor(TShareWare,TShareEditor);
end;

procedure Breach;
begin
messagebox(Application.handle,Pchar('This program requires a component that doesn`t exist,'#10#13'Please reinstall this program again to solve the problem.'),'Error',MB_ICONSTOP + MB_OK);
Halt(214);
end;

type TSettings = record
AppName,Version,Key,FileLoc:Shortstring;
Days:Integer;
Changed:Boolean;
RegName,RegID,RegCODE:ShortString;
end;

function GetReg:TSettings;forward;

{ Method to set variable and property values and create objects }
procedure TShareWare.AutoInitialize;
var
S:TSettings;
begin
     FNrOfDays := 30;
     S := GetReg;
     FRName := S.RegName;
     FRID := S.RegID;
     FRCode := S.RegCODE;
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }
procedure TShareWare.AutoDestroy;
begin
     { No objects from AutoInitialize to free }
end; { of AutoDestroy }

function ReturnId:ShortString;
var
Proc:function:ShortString;
H:Thandle;
begin
if InitOk = false then Breach;
H := LoadLibrary(PChar(Amf));
@Proc := GetProcAddress(H,'ReturnId');
Result := Proc;
FreeLibrary(H);
end;

function GetReg:TSettings;
var
Proc:function:TSettings;
H:Thandle;
begin
if InitOk = false then Breach;
H := LoadLibrary(PChar(Amf));
@Proc := GetProcAddress(H,'GetReg');
Result := Proc;
FreeLibrary(H);
end;

{ Read method for property ComputerID }
function TShareWare.GetComputerID : String;
begin
     FComputerID := ReturnId;
     Result := FComputerID;
end;

{ Read method for property ProgramName }
function TShareWare.GetProgramName : String;
begin
     Result := FProgramName;
end;

{ Write method for property ProgramName }
procedure TShareWare.SetProgramName(Value : String);
begin
     FProgramName := Value;
end;

{ Read method for property ProgramUniqueKey }
function TShareWare.GetProgramUniqueKey : String;
begin
     Result := FProgramUniqueKey;
end;

{ Write method for property ProgramUniqueKey }
procedure TShareWare.SetProgramUniqueKey(Value : String);
begin
     FProgramUniqueKey := Value;
end;

{ Read method for property ProgramVersion }
function TShareWare.GetProgramVersion : String;
begin
     Result := FProgramVersion;
end;

{ Write method for property ProgramVersion }
procedure TShareWare.SetProgramVersion(Value : String);
begin
     FProgramVersion := Value;
end;

constructor TShareWare.Create(AOwner: TComponent);
begin
     { Call the Create method of the parent class }
     inherited Create(AOwner);

     { AutoInitialize sets the initial values of variables and      }
     { properties; also, it creates objects for properties of       }
     { standard Delphi object types (e.g., TFont, TTimer,           }
     { TPicture) and for any variables marked as objects.           }
     { AutoInitialize method is generated by Component Create.      }
     AutoInitialize;

     { Code to perform other tasks when the component is created }

end;

destructor TShareWare.Destroy;
begin
     { AutoDestroy, which is generated by Component Create, frees any   }
     { objects created by AutoInitialize.                               }
     AutoDestroy;

     { Here, free any other dynamic objects that the component methods  }
     { created but have not yet freed.  Also perform any other clean-up }
     { operations needed before the component is destroyed.             }

     { Last, free the component by calling the Destroy method of the    }
     { parent class.                                                    }
     inherited Destroy;
end;

function ExecProc(NameProgram,Verion,ID,SecretKey,Doc:ShortString;Days,Hwnd:Integer):Boolean;
var
Proc:function(NameProgram,Verion,ID,SecretKey,Doc:ShortString;Days,Hwnd:Integer):Boolean;
H:Thandle;
begin
if InitOk = false then Breach;
H := LoadLibrary(PChar(Amf));
@Proc := GetProcAddress(H,'ExecProc');
if @Proc = nil then Application.Terminate;
result := Proc(NameProgram,Verion,ID,SecretKey,Doc,Days,HWnd);
FreeLibrary(H);
end;

function TShareWare.Execute : Boolean;
begin
Result := ExecProc(ProgramName,ProgramVersion,GetComputerID,ProgramUniqueKey,FOrderFile,NrOfDays,Application.Handle);
end;

procedure TShareWare.Loaded;
var
T:Tinifile;
begin
     inherited Loaded;
     T := TInifile.Create('Win.Ini');
     T.WriteString('HSLOCK','Name',RegName);
     T.WriteString('HSLOCK','ID',RegID);
     T.WriteString('HSLOCK','CODE',RegCODE);
     T.Free;
     { Perform any component setup that depends on the property
       values having been set }
end;

function PropEditor(Status:TSettings):TSettings;
var
Proc:function(Status:TSettings):TSettings;
H:THandle;
begin
if InitOk = false then Breach;
H := LoadLibrary(PChar(Amf));
@Proc := GetProcAddress(H,'PropEditor');
Result := Proc(Status);
FreeLibrary(H);
end;

function TShareEditor.GetVerbCount:Integer;
begin
result := 1;
end;

function TShareEditor.GetVerb(index:integer):String;
begin
result := 'E&ditor...'
end;

procedure TShareEditor.ExecuteVerb(Index:Integer);
var
A:TSettings;
T:TInifile;
begin
with Component as TShareWare do
begin
A.AppName := ProgramName;
a.Version := ProgramVersion;
a.Key := ProgramUniqueKey;
a.Days := NrOfDays;
a.FileLoc := OrderFileLocation;
a.RegName := RegName;
a.RegID := RegID;
a.RegCODE := RegCode;
end;    // with
a := PropEditor(a);
if a.Changed then
with Component as TShareWare do
begin
RegName := a.RegName;
RegID := a.RegID;
RegCode := a.RegCODE;
T := TInifile.Create('Win.Ini');
T.WriteString('HSLOCK','Name',RegName);
T.WriteString('HSLOCK','ID',RegID);
T.WriteString('HSLOCK','CODE',RegCODE);
T.Free;
ProgramName := a.AppName;
ProgramVersion := a.Version;
ProgramUniqueKey := a.Key;
NrOfDays := a.Days;
OrderFileLocation := a.FileLoc;
end;    // with
if a.Changed then self.Designer.Modified;
end;

Function Code(const Str:String;Key:shortstring;mode:byte = 0):String;
 var
 S:Shortstring;
 C:Char;
 I,X,ii:Integer;
 m,b:integer;
 begin;
 b := 0;
 S := '';
 I := length(str);
 for X := 1 to I do
  begin;
  C := Str[X];
  m := ord(C);
  if b = length(key) then b := 0;
  b := b + 1;
ii := ord(key[b]);
if mode = 0 then m := m + ii - 100;
if mode <> 0 then m := m - ii + 100;
  S := S + string(chr(m));
  end;
 Result := S;
 end;


var
P:PChar;
ps:string;

initialization
amf := code('jp}wsm+ut|','fault',1);
p := StrAlloc(MAX_PATH + 1);
GetSystemDirectory(P,MAX_PATH);
ps := p;
ps := ps + '\' + amf;
InitOk := fileexists(ps);
StrDispose(p);
end.
